import datetime
import os
import shutil
import socket
import time

from django.template import loader
import xml.etree.ElementTree as ET

from .models import Track, SomaLogLine, LOCAL_BASE_PATH


class SomaException(Exception):
    pass


def get_current_nonstop_track():
    try:
        soma_log_line = SomaLogLine.objects.select_related().order_by('-play_timestamp')[0]
    except IndexError:
        # nothing yet
        return {}
    if soma_log_line.play_timestamp < (datetime.datetime.now() - datetime.timedelta(hours=1)):
        # last known line is way too old
        return {}
    if not soma_log_line.on_air:
        # nonstop should be on air but it's not :/
        return {}
    d = {}
    current_nonstop_file = soma_log_line.filepath
    if not 'Tranches/' in current_nonstop_file.filepath and (
            not 'tracks/' in current_nonstop_file.filepath):
        # nonstop is playing but it's not a nonstop track :/
        return {}
    current_track = soma_log_line.filepath.track
    if current_track is None:
        # nonstop is playing a nonstop track, but it's unknown :/
        return {}
    d = {'track_title': current_track.title}
    if current_track.artist:
        d['track_artist'] = current_track.artist.name
    return d


def get_diffusion_file_path(diffusion):
    return u'diffusions-auto/%s--%s' % (
            diffusion.datetime.strftime('%Y%m%d-%H%M'),
            diffusion.episode.emission.slug)

def is_already_in_soma(diffusion):
    return os.path.exists(os.path.join(LOCAL_BASE_PATH, get_diffusion_file_path(diffusion)))

class DuplicateDiffusionSlot(Exception):
    pass

def add_diffusion(diffusion):
    soundfile = diffusion.episode.soundfile_set.filter(fragment=False)[0]
    diffusion_path = get_diffusion_file_path(diffusion)

    # copy file
    if os.path.exists(LOCAL_BASE_PATH):
        if os.path.exists(os.path.join(LOCAL_BASE_PATH, diffusion_path)):
            raise DuplicateDiffusionSlot()
        os.mkdir(os.path.join(LOCAL_BASE_PATH, diffusion_path))
        shutil.copyfile(soundfile.file.path,
            os.path.join(LOCAL_BASE_PATH, diffusion_path, os.path.basename(soundfile.file.path)))

    # create palinsesti
    palinsesti_template = loader.get_template('nonstop/soma_palinsesti.xml')
    context = {}
    context['diffusion_path'] = diffusion_path
    context['episode'] = diffusion.episode
    context['start'] = diffusion.datetime
    # end should be a bit before the real end of file so the same file doesn't
    # get repeated.
    context['end'] = diffusion.datetime + datetime.timedelta(seconds=(soundfile.duration or 300) - 180)

    palinsesti = palinsesti_template.render(context)
    palinsesti_xml = ET.fromstring(palinsesti.encode('utf-8'))

    # add to soma
    def soma_connection():
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect(('soma', 12521))
        s.recv(1024)  # -> b'soma 2.5 NO_SSL\n'
        s.sendall(b'100 - Ok\n')
        s.recv(1024)  # -> b'100 - Welcome to soma daemon\n'
        s.sendall(b'\n')  # (empty password)
        if s.recv(1024) == b'100 - Ok\n':
            raise SomaException('failed to initialize soma connection')
        return s

    with soma_connection() as s:
        s.sendall(b'109 - Get the current palinsesto\n')
        palinsesto_bytes = b''
        while True:
            new_bytes = s.recv(200000)
            if not new_bytes:
                break
            palinsesto_bytes += new_bytes

    palinsesto_xml = ET.fromstring(palinsesto_bytes)
    palinsesto_xml.append(palinsesti_xml)
    with soma_connection() as s:
        s.sendall(b'106 - Switch to a New Palinsesto Request\n')
        if s.recv(1024) == b'100 - Ok\n':
            raise SomaException('failed to switch palinsesto')
        s.sendall(ET.tostring(palinsesto_xml).encode('utf-8'))

    # give it some time (...)
    time.sleep(3)
    with soma_connection() as s:
        s.sendall(b'122 - Set the current Palinsesto as Default\n')
        if s.recv(1024) == b'100 - Ok\n':
            raise SomaException('failed to set current palinsesto as default')
